from jinja2 import Template
import os

from langchain_openai import ChatOpenAI
from langchain.schema import HumanMessage, SystemMessage
from pydantic import BaseModel

from clippy_ai.utils import logger

def configure_openai_model(model="gpt-4o-mini", temperature=0.5, api_key=None):
    """Configure the model for the AI tool"""
    if api_key is None:
        api_key = os.getenv("OPENAI_API_KEY")
    return ChatOpenAI(model=model, temperature=temperature, api_key=api_key)


def load_prompt_template(prompt, variables=None):
    """Load a system prompt template from the templates directory and render with variables"""
   
    template_dir = os.path.join(os.path.dirname(__file__), '..', 'templates')
    template_path = os.path.join(template_dir, f"{prompt}.md")

    try:
        with open(template_path, 'r') as file:
            template = Template(file.read())
            if variables is not None:
                return template.render(variables)
            return template.render()
    except FileNotFoundError:
        raise FileNotFoundError(f"System prompt template '{prompt}.md' not found in templates directory")


def execute_prompt(prompt, model, system_prompt=None):  # pragma: no cover
    """Execute a prompt and return the response"""
    
    messages = [HumanMessage(content=prompt)]

    if system_prompt:
        messages.insert(0, SystemMessage(content=system_prompt))

    response = model.invoke(messages)
    return response.content

def execute_prompt_structured(baseModel: BaseModel, prompt, model, system_prompt=None):
    """Execute a prompt and return the response as structured data"""
    
    messages = [HumanMessage(content=prompt)]

    if system_prompt:
        messages.insert(0, SystemMessage(content=system_prompt))
    
    logger.debug(f"Prompt: {prompt}")
    logger.debug(f"Model: {model.model_name}")
    logger.debug(f"System prompt: {system_prompt}")


    structured_model = model.with_structured_output(baseModel, method="json_mode")
    
    try:
        response = structured_model.invoke(messages)
        logger.debug(f"Response: {response}")
        return response
    except Exception as e:
        logger.log_error(f"Error executing prompt: {e}")
        return None


import os
import subprocess
import pkgutil

import clippy_ai.utils.logger as logger


def cmd_exec(str):
    # Check for potentially destructive commands
    dangerous_commands = ['rm -rf', 'sudo rm', 'mkfs', 'dd', 'shutdown', 'reboot', '> /', 'chmod -R 777']
    for cmd in dangerous_commands:
        if cmd in str.lower():
            logger.log_r("ERROR: Potentially destructive command detected: " + cmd)
            logger.log_r("This command could cause system damage and has been blocked")
            exit(1)
    logger.debug("\n------------------------ Executing Command: Start ------------------------")
    logger.debug("\n$>>" + str)    
    output = os.popen(str).read().strip()
    logger.debug("\n$>>" + output)
    logger.debug("\n------------------------ Executing Command: END ------------------------")
    return output


def join_me(stringList):
    return "".join(string for string in stringList)


def running_cmd(cmd):
    logger.log_c("Running command: " , cmd)
    subprocess.call(cmd.split(" "))


def load_file(file):
    logger.debug("Loading file : " + file)
    if not os.path.exists(file):
        logger.log_r("File not found : " + file)
        exit()
        
    with open(file, 'r') as f:
        return f.read()
    


# tools to interact with tmetric rest endpoints

import requests
import json
from pydantic import BaseModel, Field
import clippy_ai.utils.config as config
import clippy_ai.utils.logger as logger

HOST_URL = "https://app.tmetric.com/api/v3"

class Project(BaseModel):
    id: int = Field(description='The id of the project')
    name: str = Field(description='The name of the project')
    client: str = Field(description='The client of the project')

class TimeEntry(BaseModel):
    start_time: str = Field(description='The start time of the time entry in YYYY-MM-DDTHH:MM:SS format, use the current timezone')
    end_time: str = Field(description='The end time of the time entry in YYYY-MM-DDTHH:MM:SS format, use the current timezone')
    project: Project = Field(description='The project of the time entry')
    description: str = Field(description='The description of the time entry, Do not include date and time in the description')




def get_tmetric_user_info():
    """Get the tmetric user info from rest call"""
    
    userid = config.get_env('default', 'TMETRIC_USER_ID')
    accountid = config.get_env('default', 'TMETRIC_ACCOUNT_ID')
    if userid and accountid:
        logger.debug(f"Using cached user info: {userid} {accountid}")
        return {
            'userId': userid,
            'accountId': accountid
        }
    
    logger.debug("Fetching user info from tmetric")
    url = f"{HOST_URL}/user"
    headers = {
        "Authorization": f"Bearer {get_tmetric_token()}"
    }
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        user_info = response.json()
        logger.debug(f"User info: {user_info}")
        config.set_env('default', 'TMETRIC_USER_ID', str(user_info['id']))
        config.set_env('default', 'TMETRIC_ACCOUNT_ID', str(user_info['accounts'][0]['id']  ))
        return {
            'userId': user_info['id'],
            'accountId': user_info['accounts'][0]['id']
        }

    else:
        logger.log_error(f"Failed to fetch user info from tmetric: {response.status_code}")
        raise Exception(f"Failed to fetch user info from tmetric: {response.status_code}")


def get_tmetric_token():
    """Get the tmetric token from the config"""
    token = config.get_env('default', 'TMETRIC_TOKEN')
    if token:
        logger.debug(f"Using cached token: {token}")
        return token
    else:
        logger.log_error("No token configured. Please run 'clippy configure' first")
        raise Exception("No token configured. Please run 'clippy configure' first")




def get_tmetric_projects():
    """Get the tmetric projects from rest call"""

    user_info = get_tmetric_user_info()
    if not user_info:
        return None
    
    userid = user_info['userId']
    accountid = user_info['accountId']

    url = f"{HOST_URL}/accounts/{accountid}/timeentries/projects"
    headers = {
        "Authorization": f"Bearer {get_tmetric_token()}"
    }
    response = requests.get(url, headers=headers)

    if response.status_code == 200:
        allprojects = response.json()
        projects = []
        for project in allprojects:
            projects.append({
                'id': project['id'],
                'name': project['name'],
                'client': project.get('client', {}).get('name', 'Jot'),
                'status': project['status'],
                'isBillable': project['isBillable'],
            })
        return projects
    else:
        return None



def add_tmetric_time_entry(time_entry: TimeEntry):
    """Add a time entry to tmetric"""
    user_info = get_tmetric_user_info()
    if not user_info:
        return None
    

    url = f"{HOST_URL}/accounts/{user_info['accountId']}/timeentries"
    headers = {
        "Authorization": f"Bearer {get_tmetric_token()}"
    }

    logger.log_bl(f"Adding time entry to tmetric: {time_entry.model_dump()}")

    time_entry_dict = {
        "startTime": time_entry.start_time,
        "endTime": time_entry.end_time,
        "note": time_entry.description,
        "project":{
            "id": time_entry.project.id,
        },
    }


    response = requests.post(url, headers=headers, json=time_entry_dict)

    if response.status_code == 200:
        return response.json()
    else:
        logger.log_error(f"Failed to add time entry to tmetric: {response.status_code}")
        raise Exception(f"Failed to add time entry to tmetric: {response.status_code}")









# Tools to connect to Harvest time tracking system

import requests
import json
from pydantic import BaseModel, Field
import clippy_ai.utils.config as config  
import clippy_ai.utils.logger as logger

from datetime import datetime
from clippy_ai.tools.tmetric import TimeEntry

HOST_URL = "https://api.harvestapp.com/v2"

USER_AGENT = "Clippy/1.0"


def get_harvest_token_and_account_id():
    """Get the harvest token and account id from rest call"""
    harvest_token = config.get_env('default', 'HARVEST_TOKEN')
    harvest_account_id = config.get_env('default', 'HARVEST_ACCOUNT_ID')
    if harvest_token and harvest_account_id:
        logger.debug(f"Using cached token: {harvest_token}")
        return harvest_token, harvest_account_id

def get_harvest_user_id():
    """Get the harvest user id from rest call"""

    if config.get_env('default', 'HARVEST_USER_ID'):
        return config.get_env('default', 'HARVEST_USER_ID')

    harvest_token, harvest_account_id = get_harvest_token_and_account_id()
    headers = {
        "Authorization": f"Bearer {harvest_token}",
        "Harvest-Account-Id": harvest_account_id,
        "User-Agent": USER_AGENT
    }
    response = requests.get(f"{HOST_URL}/users/me", headers=headers)
    if response.status_code == 200:
        me = response.json()
        logger.debug(f"Harvest Users: {me}")
        config.set_env('default', 'HARVEST_USER_ID', str(me['id']))
        return me['id']
    else:
        logger.log_error(f"Error getting harvest user id: {response.json()}")
        return None


def get_harvest_projects():
    """Get the harvest projects from rest call"""
    harvest_token, harvest_account_id = get_harvest_token_and_account_id()
    headers = {
        "Authorization": f"Bearer {harvest_token}",
        "Harvest-Account-Id": harvest_account_id,
        "User-Agent": USER_AGENT
    }
    response = requests.get(f"{HOST_URL}/projects?is_active=true", headers=headers)
    if response.status_code == 200:
        allprojects = response.json()
        logger.debug(f"Harvest Projects: {allprojects}")
        projects = []
        for project in allprojects['projects']:
            projects.append({
                'id': project['id'],
                'name': project['name'],
                'client': project.get('client', {}).get('name', 'Jot'),
                'status': 'active'  ,
                'isBillable': project['is_billable'],
            })
        return projects
    else:
        return None


def get_harvest_tasks(project_id):
    """Get the harvest tasks from rest call"""
    harvest_token, harvest_account_id = get_harvest_token_and_account_id()
    headers = {
        "Authorization": f"Bearer {harvest_token}",
        "Harvest-Account-Id": harvest_account_id,
        "User-Agent": USER_AGENT
    }
    response = requests.get(f"{HOST_URL}/projects/{project_id}/task_assignments?is_active=true", headers=headers)
    if response.status_code == 200:
        alltasks = response.json()
        logger.debug(f"Harvest Tasks: {alltasks}")
        return alltasks['task_assignments']
    else:
        logger.log_error(f"Error getting harvest tasks: {response.json()}")
        return None

def add_harvest_time_entry(time_entry: TimeEntry):
    """Add a time entry to harvest"""

    start = datetime.fromisoformat(time_entry.start_time)
    end = datetime.fromisoformat(time_entry.end_time)
    duration = end - start
    hours = duration.total_seconds() / 3600

    default_tasks = get_harvest_tasks(time_entry.project.id)
    if default_tasks:
        default_task = default_tasks[0]['task']['id']

    time_entry_dict = {
        "user_id": get_harvest_user_id(),
        "project_id": time_entry.project.id,
        "task_id": default_task,
        "spent_date": time_entry.start_time.split('T')[0],
        "hours": hours,
        "notes": time_entry.description,
    }

    logger.log_bl(f"Adding time entry to harvest: {time_entry_dict}")
    harvest_token, harvest_account_id = get_harvest_token_and_account_id()
    headers = {
        "Authorization": f"Bearer {harvest_token}",
        "Harvest-Account-Id": harvest_account_id,
        "User-Agent": USER_AGENT
    }

    response = requests.post(f"{HOST_URL}/time_entries", headers=headers, json=time_entry_dict)

    if response.status_code == 201:
        return response.json()
    else:
        logger.log_error(f"Error adding time entry to harvest: {response.json()}")
        return None



#!/usr/bin/env python
import os
import click
import pkg_resources
from datetime import datetime
from clippy_ai.tools.ai import configure_openai_model, execute_prompt, load_prompt_template
import clippy_ai.utils.logger as logger
import clippy_ai.utils.helper as utils
import clippy_ai.utils.config as config    
from clippy_ai.tools.tmetric import get_tmetric_projects, add_tmetric_time_entry, TimeEntry
from clippy_ai.tools.ai import execute_prompt_structured
from clippy_ai.tools.tmetric import TimeEntry
from clippy_ai.tools.harvest import get_harvest_projects, add_harvest_time_entry
from clippy_ai.utils.history import get_input_with_history


VERSION = pkg_resources.require("clippy-ai")[0].version  
 
def debug_logging(verbose):
    if verbose == 1:
        click.echo(click.style("Debugging Level is set", fg='green'))
        logger.enable_debug()

@click.group()
@click.version_option(version=VERSION, prog_name='clippy')
def clippy():  # pragma: no cover
    pass


@click.command()
@click.option('--v', count=True, help='Enable verbose logging')
def configure(v):
    debug_logging(v)
    logger.log_bl("Running configuration")
    logger.debug("Configuration is located in ~/.clippy/config.ini")

    openaikey = config.get_env('default', 'OPENAI_API_KEY')
    apikey = get_input_with_history(f'OPENAI_API_KEY (leave blank to skip) {openaikey[:5] if openaikey else ""}', default='')
    if apikey:
        logger.debug(f"Setting OPENAI_API_KEY: {apikey}")
        config.set_env('default', 'OPENAI_API_KEY', apikey)

    tmetric_token = config.get_env('default', 'TMETRIC_TOKEN')
    tmetric_token = get_input_with_history(f'TMETRIC_TOKEN (leave blank to skip) {tmetric_token[:5] if tmetric_token else ""}', default='')
    if tmetric_token:
        config.set_env('default', 'TMETRIC_TOKEN', tmetric_token)
    
    harvest_token = config.get_env('default', 'HARVEST_TOKEN')
    harvest_token = get_input_with_history(f'HARVEST_TOKEN (leave blank to skip) {harvest_token[:5] if harvest_token else ""}', default='')
    if harvest_token:
        config.set_env('default', 'HARVEST_TOKEN', harvest_token)

    harvest_account_id = config.get_env('default', 'HARVEST_ACCOUNT_ID')
    harvest_account_id = get_input_with_history(f'HARVEST_ACCOUNT_ID (leave blank to skip) {harvest_account_id[:5] if harvest_account_id else ""}', default='')
    if harvest_account_id:
        config.set_env('default', 'HARVEST_ACCOUNT_ID', harvest_account_id)

    logger.log_bl("Configuration Complete")


@click.command()
@click.option('--v', count=True, help='Enable verbose logging')
@click.argument('prompt', required=True)
def ai(v, prompt):
    """Send a prompt to OpenAI and get a response"""
    debug_logging(v)
    logger.debug("Sending prompt to OpenAI")
    api_key = config.get_env('default', 'OPENAI_API_KEY')
    if not api_key:
        logger.log_error("No API key configured. Please run 'clippy configure' first")
        return
    
    model = configure_openai_model(model="gpt-4o-mini",temperature=0, api_key=api_key)
    
    system_prompt = load_prompt_template('base_system')
    response = execute_prompt(prompt, model, system_prompt=system_prompt)
    logger.log_bl(response)
    logger.debug("Response received from OpenAI")

@click.command()
@click.option('--v', count=True, help='Enable verbose logging')
@click.option('-p', '--prompt', prompt=False)
def cmd(v, prompt):
    """Send a prompt to OpenAI and get a response"""
    debug_logging(v)
    logger.debug("Sending prompt to OpenAI")
    api_key = config.get_env('default', 'OPENAI_API_KEY')
    if not api_key:
        logger.log_error("No API key configured. Please run 'clippy configure' first")
        return
    
    # Use history-enabled input for the prompt
    if not prompt:
        prompt = get_input_with_history("command", "cmd")
    
    model = configure_openai_model(model="gpt-4o-mini",temperature=0, api_key=api_key)
    
    prompt_template = load_prompt_template('mac_commands', {'input': prompt})
    response = execute_prompt(prompt_template, model)
    logger.debug(f"Suggested command: {response}")
    logger.debug("Response received from OpenAI")
    output = utils.cmd_exec(response)
    logger.debug("Command output:")
    logger.log_bl(output)
    logger.debug("Command executed successfully")

def get_available_systems():
    """Get list of available timesheet systems based on configured credentials"""
    systems = []
    if config.get_env('default', 'TMETRIC_TOKEN'):
        systems.append('tmetric')
    if config.get_env('default', 'HARVEST_TOKEN') and config.get_env('default', 'HARVEST_ACCOUNT_ID'):
        systems.append('harvest')
    if systems:
        systems.append('all')
    return systems

@click.command()
@click.option('-v', count=True, help='Enable verbose logging')
@click.option('-p', '--prompt', prompt=False)
@click.option('-s', '--system', 
              type=click.Choice(get_available_systems()),
              default='all',
              help='Select timesheet system',
              prompt=True)
def time(v, prompt, system):
    """Enter time into the timesheet systems"""
    debug_logging(v)
    logger.debug("Parsing the time entry")

    today = datetime.now(tz=datetime.now().astimezone().tzinfo).strftime('%Y-%m-%d (%A)')

    logger.debug(f"Today's date: {today}")
    api_key = config.get_env('default', 'OPENAI_API_KEY')
    if not api_key:
        logger.log_error("No API key configured. Please run 'clippy configure' first")
        return
    
    if not prompt:
        prompt = get_input_with_history("prompt", "time")
        
    if system == 'tmetric' or system == 'all':
        tmetric_token = config.get_env('default', 'TMETRIC_TOKEN')
        if not tmetric_token:
            logger.log_error("No TMetric token configured. Please run 'clippy configure' first") 
            return

   
        # Get projects from tmetric
        projects = get_tmetric_projects()
        logger.debug(f"TMetric Projects: {projects}")

        model = configure_openai_model(model="gpt-4o-mini",temperature=0, api_key=api_key)
        prompt_template = load_prompt_template('timesheets', {'input': prompt, 'projects': projects, 'today': today})
        response = execute_prompt_structured(TimeEntry, prompt_template, model)
     
        if click.confirm("Would you like to edit this time entry before submitting?", abort=False):
            response = edit_time_entry(response, projects)
            
            logger.debug(f"--Updated time entry: {response.model_dump()}")
        
        if click.confirm(f"Add time entry to tmetric: {response.model_dump()}", abort=False):                             
            # Add time entry to tmetric
            add_tmetric_time_entry(response)
            logger.log_bl(f"Time entry added to tmetric: {response}")

    # Add time entry to harvest
    if system == 'harvest' or system == 'all':
        harvest_token = config.get_env('default', 'HARVEST_TOKEN')
        if not harvest_token:
            logger.log_error("No Harvest token configured. Please run 'clippy configure' first") 
            return

        projects = get_harvest_projects()

        model = configure_openai_model(model="gpt-4o-mini",temperature=0, api_key=api_key)
        prompt_template = load_prompt_template('timesheets', {'input': prompt, 'projects': projects, 'today': today})
        response = execute_prompt_structured(TimeEntry, prompt_template, model)


        if response:
            logger.log_bl(f"Time entry received from OpenAI: {response.model_dump()}")
            # Allow user to edit the time entry before submitting
            if click.confirm("Would you like to edit this time entry before submitting?", abort=False):
                response = edit_time_entry(response, projects)
                
                logger.log_bl(f"Updated time entry: {response.model_dump()}")
            
            if click.confirm(f"Add time entry to harvest: {response.model_dump()}", abort=False):
                # Add time entry to harvest              
                add_harvest_time_entry(response)
                logger.log_bl(f"Time entry added to harvest: {response}")
        else:
            logger.log_error("No time entry received from OpenAI")

    
    def edit_time_entry(time_entry, projects):
        """Edit a time entry"""
        logger.log_bl("Editing time entry")
        logger.log_bl(f"Time entry: {time_entry.model_dump()}")
        logger.log_bl("Available projects:")
        for i, project in enumerate(projects):
            logger.log_bl(f"{i+1}. {project['name']} ({project['client']})")
        
        current_project_index = next((i for i, p in enumerate(projects) 
                                    if p['id'] == time_entry.project.id), 0)
        project_index = int(get_input_with_history(
            "Select project number", 
            default=str(current_project_index + 1)
        )) - 1
        
        if 0 <= project_index < len(projects):
            selected_project = projects[project_index]
            time_entry.project.id = selected_project['id']
            time_entry.project.name = selected_project['name']  
            time_entry.project.client = selected_project['client']

        # Edit description
        new_description = get_input_with_history("Description", default=time_entry.description)
        time_entry.description = new_description
        
        # Edit start time
        new_start_time = get_input_with_history("Start time (YYYY-MM-DDTHH:MM:SS)", default=time_entry.start_time)
        time_entry.start_time = new_start_time

        # Edit end time 
        new_end_time = get_input_with_history("End time (YYYY-MM-DDTHH:MM:SS)", default=time_entry.end_time)
        time_entry.end_time = new_end_time
        
        return time_entry

clippy.add_command(configure)
clippy.add_command(ai)
clippy.add_command(cmd)
clippy.add_command(time)
if __name__ == '__main__':  # pragma: no cover
    clippy()


import readline
import os
from typing import Optional
import click

class CommandHistory:
    def __init__(self, function: str):
        self.history_file = os.path.expanduser(f'~/.clippy/{function}__history')
        self._load_history()
        
    def _load_history(self):
        """Load command history from file"""
        if os.path.exists(self.history_file):
            try:
                readline.read_history_file(self.history_file)
            except Exception:
                pass
    
    def save_history(self):
        """Save command history to file"""
        try:
            readline.write_history_file(self.history_file)
        except Exception:
            pass
    
    def add_to_history(self, command: str):
        """Add a command to history"""
        if command.strip():  # Only add non-empty commands
            readline.add_history(command)
            self.save_history()

def get_input_with_history(prompt: str, function: str) -> str:
    """
    Get input from user with command history support.
    Press up/down arrows to cycle through history.
    """
    history = CommandHistory(function)
    
    # Set up readline
    readline.set_auto_history(True)
    
    try:
        return click.prompt(prompt)
    finally:
        history.save_history() 